@@ -82,9 +82,19 @@ class Tiled(Data):
         """
 
         tiles = super().__new__(cls, tiles, job, otype, ALLOWED_TILED_TYPES)
+        tiles.parent = tiles
+        tiles.slices = []
 
         return tiles
 
+    @property
+    def s(self):
+
+        """ Get the Slice object for tile-wise slicing.
+        """
+
+        return Slice(self)
+
 
 class Stitched(Data):
 
@@ -98,7 +108,7 @@ class Stitched(Data):
         Parameters
         ----------
             stitched : numpy.ndarray or Stitched
-                Stitched array.
+                Stitched object.
             job : Job
                 Job that generated this stitched object.
             otype : str
@@ -107,9 +117,50 @@ class Stitched(Data):
         Returns
         -------
             stitched : Stitched
-                Stitched array.
+                Stitched object.
         """
 
         stitched = super().__new__(cls, stitched, job, otype, ALLOWED_STITCHED_TYPES)
 
         return stitched
+
+
+class Slice:
+
+    """ Slice class for tile-wise slicing.
+
+    Parameters
+    ----------
+        tiles : numpy.ndarray or Tiled
+            Array of tiles.
+    """
+
+    def __init__(self, tiles):
+
+        self.tiles = tiles
+
+    def __getitem__(self, slices):
+
+        """ Apply slices to each tile.
+
+        Parameters
+        ----------
+            slices : tuple, optional, default (slice(None))
+                Tuple of slice objects designating slices to be extracted.
+
+        Returns
+        -------
+            sliced_tiles : Tiled
+                Stitched object.
+        """
+
+        sliced_tiles = self.tiles.copy()
+        sliced_tiles.parent = self.tiles.parent
+        sliced_tiles.slices = self.tiles.slices + [slices]
+        nonempty_indices = self.tiles.profile.nonempty_indices
+        nonempty_tiles = sliced_tiles[tuple(zip(*nonempty_indices))]
+
+        for index, tile in zip(nonempty_indices, nonempty_tiles):
+            sliced_tiles[index] = tile[slices]
+
+        return sliced_tiles
